import React, { ReactNode } from "react";
import { VirtualListHandle } from "react-base-virtual-list";
export type Id = string | number;
export interface Stat<T> {
    id: Id;
    pid: Id | null;
    childIds: Id[];
    siblingIds: Id[];
    index: number;
    level: number;
    node: T;
    parent: T | null;
    parentStat: Stat<T> | null;
    children: T[];
    childStats: Stat<T>[];
    siblings: T[];
    siblingStats: Stat<T>[];
    _isStat?: boolean;
    open: boolean;
    checked: boolean;
    draggable: boolean;
}
export type NodeAttrs = {
    'data-key': string;
    'data-level': string;
    'data-node-box': boolean;
    'data-drag-placeholder'?: boolean;
    'data-dragging'?: boolean;
} & React.HTMLProps<HTMLDivElement>;
export declare const defaultProps: {
    /**
     *
     */
    idKey: string;
    parentIdKey: string;
    childrenKey: string;
    indent: number;
    dragOpen: boolean;
    dragOpenDelay: number;
    placeholderId: string;
    dataType: "flat" | "tree";
    direction: "ltr" | "rtl";
    rootId: Id | null;
    virtual: boolean;
};
export interface HeTreeProps<T extends Record<string, any>> extends Partial<typeof defaultProps> {
    data: T[];
    isFunctionReactive?: boolean;
    keepPlaceholder?: boolean;
    renderNode?: (stat: Stat<T>) => ReactNode;
    renderNodeBox?: (info: {
        stat: Stat<T>;
        attrs: NodeAttrs;
        isPlaceholder: boolean;
    }) => ReactNode;
    canDrag?: (stat: Stat<T>) => boolean | null | undefined | void;
    canDrop?: (stat: Stat<T>, index?: number) => boolean | null | undefined | void;
    canDropToRoot?: (index?: number) => boolean;
    customDragImage?: (e: React.DragEvent<HTMLElement>, stat: Stat<T>) => void;
    onDragStart?: (e: React.DragEvent<HTMLElement>, stat: Stat<T>) => void;
    onDragOver?: (e: React.DragEvent<HTMLElement>, stat: Stat<T>, isExternal: boolean) => void;
    onExternalDragOver?: (e: React.DragEvent<HTMLElement>) => boolean;
    onExternalDrop?: (e: React.DragEvent<HTMLElement>, parentStat: Stat<T> | null, index: number) => void;
    onDragEnd?: (e: React.DragEvent<HTMLElement>, stat: Stat<T>, isOutside: boolean) => void | boolean;
    onChange: (data: T[]) => void;
    onDragOpen?: (stat: Stat<T>) => void;
    openIds?: Id[];
    checkedIds?: Id[];
}
export declare function useHeTree<T extends Record<string, any>>(props0: HeTreeProps<T>): {
    visibleIds: Id[];
    attrsList: NodeAttrs[];
    virtualListRef: React.RefObject<VirtualListHandle>;
    draggingStat: Stat<T> | undefined;
    dragOverStat: Stat<T> | undefined;
    placeholder: {
        parentStat: Stat<T> | null;
        level: number;
        index: number;
    } | null | undefined;
    renderTree: (options?: {
        className?: string;
        style?: React.CSSProperties;
    }) => ReactNode;
    renderHeTree: (options?: {
        className?: string;
        style?: React.CSSProperties;
    }) => ReactNode;
    scrollToNode: (idOrNodeOrStat: Id | T | Stat<T>, block?: 'start' | 'end' | 'center' | 'nearest') => boolean;
    rootIds: Id[];
    rootNodes: T[];
    rootStats: Stat<T>[];
    allIds: Id[];
    getStat: (idOrNodeOrStat: T | Stat<T> | Id) => Stat<T>;
};
export type WalkTreeDataYield<T> = [
    T,
    {
        parent: T | null;
        parents: T[];
        siblings: T[];
        index: number;
        skipChildren: VoidFunction;
        exitWalk: VoidFunction;
    }
];
/**
 * example: walkTreeData(treeData, 'children', (node, info)=> {})
 */
export declare function walkTreeData<T extends Record<Id, any>>(treeData: T[], handler: (...args: WalkTreeDataYield<T>) => void, childrenKey?: string): void;
/**
 * example: for (const [node, info] of walkTreeDataGenerator(treeData, 'children')) {...}
 */
export declare function walkTreeDataGenerator<T extends Record<Id, any>>(treeData: T[], childrenKey?: string): Generator<WalkTreeDataYield<T>>;
export declare function findTreeData<T extends Record<Id, any>>(treeData: T[], handler: (...args: WalkTreeDataYield<T>) => void | boolean | any, childrenKey?: string): T | undefined;
export declare function filterTreeData<T extends Record<Id, any>>(treeData: T[], handler: (...args: WalkTreeDataYield<T>) => void | boolean | any, childrenKey?: string): T[];
export declare function walkParentsGenerator<T>(node: T, parentKeyOrGetter: string | ((node: T) => T | void | undefined | null), options?: {
    withSelf: boolean;
}): Generator<NonNullable<T>, void, unknown>;
export declare function sortFlatData<T extends Record<string, any>>(data: T[], options0?: Partial<typeof flatDataDefaultOptions>): T[];
declare const flatDataDefaultOptions: {
    idKey: string;
    parentIdKey: string;
};
export type WalkFlatDataYield<T> = [
    T,
    {
        parent: T | null;
        parents: T[];
        index: number;
        treeIndex: number;
        id: Id;
        pid: Id | null;
        skipChildren: VoidFunction;
        exitWalk: VoidFunction;
    }
];
export declare function walkFlatDataGenerator<T extends Record<Id, any>>(flatData: T[], options0?: Partial<typeof flatDataDefaultOptions>): Generator<WalkFlatDataYield<T>>;
export declare function walkFlatData<T extends Record<Id, any>>(flatData: T[], handler: (...args: WalkFlatDataYield<T>) => void, options?: Partial<typeof flatDataDefaultOptions>): void;
/**
 * Convert index in sibling to tree index which in flat data.
 * @param flatData
 * @param parentId null means root
 * @param indexInSiblings null means append to the end of siblings
 * @param options0
 * @returns tree index
 */
export declare function convertIndexToTreeIndexInFlatData<T extends Record<Id, any>>(flatData: T[], parentId: Id | null, indexInSiblings: Id | null, options0?: Partial<typeof flatDataDefaultOptions>): number;
export declare function addToFlatData<T extends Record<Id, any>>(flatData: T[], newNode: T, indexInSiblings: Id | null, options0?: Partial<typeof flatDataDefaultOptions>): void;
export declare function removeByIdInFlatData<T extends Record<Id, any>>(flatData: T[], removeId: Id | null, options0?: Partial<typeof flatDataDefaultOptions>): T[];
export declare function openParentsInFlatData<T extends Record<Id, any>>(flatData: T[], openIds: Id[], idOrIds: Id | Id[], options0?: Partial<typeof flatDataDefaultOptions>): Id[];
declare const treeDataDefaultOptions: {
    idKey: string;
    childrenKey: string;
};
export declare function openParentsInTreeData<T extends Record<Id, any>>(treeData: T[], openIds: Id[], idOrIds: Id | Id[], options0?: Partial<typeof treeDataDefaultOptions>): Id[];
export declare function updateCheckedInFlatData<T extends Record<Id, any>>(flatData: T[], checkedIds: Id[], idOrIds: Id | Id[], checked: boolean, options?: Partial<typeof flatDataDefaultOptions>): Id[][];
export declare function updateCheckedInTreeData<T extends Record<Id, any>>(treeData: T[], checkedIds: Id[], idOrIds: Id | Id[], checked: boolean, options0?: Partial<typeof treeDataDefaultOptions>): Id[][];
export {};
